<?php
namespace Controller;

use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Manager\Mongo\Manager;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Moment\Moment;
use Zeedhi\Framework\DTO\Response\Message;
use Cron\CronExpression;

class ImportSchedule extends Crud {

    protected $dataSourceName = 'importSchedule';
    /** @var \Service\ImportSchedule $importScheduleService */
    protected $importScheduleService;
    /** @var Session $sessionManager */
    protected $sessionManager;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\ImportSchedule $importScheduleService, Session $sessionManager) {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->importScheduleService = $importScheduleService;
        $this->sessionManager = $sessionManager;
    }

    public function find(Request\Filter $request, Response $response) {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        $filterData = new FilterCriteria($this->dataSourceName, $conditions);
        $filterData->addCondition('organizationId', (string)$this->sessionManager->get('organizationId') );

        $rows = $this->mongoManager->findBy($filterData)->getRows();
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }

    public function save(Request $request, Response $response) {
        try {
            $data = array();
            $row = $request->getRow();
            if(empty($row['period'])){
                $row['period'] = "";
            }
            if(!empty($row['dataSource'])  && !empty($row['persistType'])) {
                $schedule                       = array();
                $schedule['dataSource']         = $row['dataSource'];
                $schedule['period']             = $row['period'];
                $schedule['persistType']        = $row['persistType'];
                $schedule['scheduleExpression'] = $row['scheduleExpression'];
                $schedule['beforeImportScript'] = $row['beforeImportScript'];
                $schedule['afterImportScript']  = $row['afterImportScript'];
                $schedule['status']             = $row['status'];
                $schedule['type']               = $row['type'];
                $schedule['date'] = new \MongoDB\BSON\UTCDateTime((new Moment(null, 'America/Sao_Paulo'))->startOf('day')->format('U').'000');

                $persistedItems = $this->importScheduleService->saveOne($schedule);
                $nrPersistedRows = count($persistedItems);
                $response->addNotification(new Response\Notification("{$nrPersistedRows} row(s) persisted with success"));
            } else {
                $response->setError(new Response\Error("Missing information to save.", 400));
            }

            $response->addDataSet(new DataSet($this->dataSourceName, $data));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getRelatedImportSchedule($dataSourceId){
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('dataSource', $dataSourceId);
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }
    
    public function deleteRelatedImportSchedule(Request $request, Response $response){
        $relatedImport = $this->getRelatedImportSchedule($request->getRow()["dataSource"]);
        $dataSet = new DataSet($this->dataSourceName, $relatedImport);
        $this->mongoManager->delete($dataSet);
    }
    
}