<?php

namespace Service;

use Util\Utilities;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class DefaultColumns
{

    protected $dataSourceName = "defaultColumns";

    /** @var Manager*/
    protected $mongoManager;

    public function __construct(Manager $mongoManager)
    {
        $this->mongoManager = $mongoManager;
    }

    public function getDefaultColumns($filterData)
    {
        $filterData->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->findByPagination($filterData, $filterData->getOrderBy())->getRows();
    }
    
    public function count($filter) 
    {
	    $filterCriteria = new FilterCriteria($filter->getDataSourceName(), $filter->getConditions());
	    $filterCriteria->setPage(1);
	    $filterCriteria->setPageSize(9999);
	    return $this->mongoManager->count($filterCriteria);
	}

    public function validateFilter(&$params)
    {
        $params['limit'] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
        $params['page'] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
        
        $filter = Utilities::parseParamNextToZmartFilterCriteria($this->dataSourceName, $params);
        $conditions = $filter->getConditions();
        
        foreach ($conditions as &$condition) {
            if ($condition["value"] == "true" || $condition["value"] == "false") {
                $condition["value"] = ($condition["value"] == "true");
                $condition["operator"] = FilterCriteria::EQ;
            } else if (is_bool($condition["value"])) {
                $condition["operator"] = FilterCriteria::EQ;
            }
        }
        
        $newFilter = Utilities::rebuildFilterCriteriaWithDifferentConditions($filter, $conditions);
        
        foreach ($filter->getOrderBy() as $column => $direction) {
            $newFilter->addOrderBy($column, $direction);
        }

        return $newFilter;
    }

    public function dataSetSave(DataSet $dataSet)
    {
        $dataSet->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->persist($dataSet);
    }

    public function filterFind(FilterCriteria $filterCriteria)
    {
        $filterCriteria->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->findBy($filterCriteria);
    }

    public function findByName($name)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('name', FilterCriteria::EQ, $name);
        return $this->filterFind($filterCriteria);
    }

    public function findByNames(array $names)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('name', FilterCriteria::IN, $names);
        return $this->filterFind($filterCriteria);
    }

    public function insertNewDefault(array $column)
    {
        $dataSet = $this->findByName($column['name']);
        $oldColumns = $dataSet->getRows();
        if (empty($oldColumns)) {
            return $this->dataSetSave(new DataSet($this->dataSourceName, array($column)));
        }
    }

    public function updateOrInsertDefaultColumn(array $column)
    {
        $columnWithId = $this->setDefaultColumnIdIfExists($column);
        return $this->dataSetSave(new DataSet($this->dataSourceName, array($columnWithId)));
    }

    /**
     * @param $column
     * @return $column with _id if exists a default column with the same name
     */
    protected function setDefaultColumnIdIfExists($column)
    {
        $dataSet = $this->findByName($column['name']);
        $oldColumns = $dataSet->getRows();
        if (!empty($oldColumns)) {
            $oldColumn = $oldColumns[0];
            $column['_id'] = $oldColumn['_id'];
        }
        return $column;
    }
}
