<?php

namespace Service;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;

class PeriodsService {

    protected $dataSourceName = 'periods';
    protected $orgMongoManager;
    protected $baseMongoManager;

    public function __construct (Manager $orgMongoManager, Manager $baseMongoManager){
        $this->orgMongoManager = $orgMongoManager;
        $this->baseMongoManager = $baseMongoManager;
    }

    public function getPeriodsGeneral($params)
    {
        unset($params['page']);
        unset($params['limit']);
        $base = $this->getPeriodsFromBase($params);
        $org = $this->getPeriodsFromOrg($params);
        $return = array_merge($base, $org);
        return $return;

    }

    public function getPeriodsFromBase($params){
        $response = $this->baseMongoManager->findNext($this->dataSourceName, $params);

        foreach ($response as $key => &$row) {
            $row['base'] = true;
            $response[$key] = iterator_to_array($row);
        }

        return $response;
    }

    public function getPeriodsFromOrg($params){
        $response = $this->orgMongoManager->findNext($this->dataSourceName, $params);
        return $response;
    }



    public function getDateByDateExpression($dateExpression){
        $result = array();
        foreach ($dateExpression as $expression) {
            $result[] = PeriodMathExpression\Period::getDate($expression);
        }
        return $result;

    }

    public function findByCriteria($manager,$columnName, $value, $operator = '=') {
        $criteria = array( array(
            "columnName" => $columnName,
            "value" => $value,
            "operator" => $operator
        ));

        $filterCriteria = new FilterCriteria($this->dataSourceName, $criteria);
        return $manager->findBy($filterCriteria)->getRows();
    }

    public function findById($idPeriod) {
        $period = $this->findByCriteria($this->orgMongoManager,'_id', $idPeriod);
        if(empty($period)){
            $period = $this->findByCriteria($this->baseMongoManager,'_id', $idPeriod);
        }
        return count($period) > 0 ? $period[0] : null;
    }

    private function validateSearch($params){
        $search = array();
        if(array_key_exists("search", $params)){
            array_push($search, array(
                "columnName" => 'name|labelOnView|START_DATE_EXPRESSION|END_DATE_EXPRESSION|periodsCode',
                "value" => $params['search'],
                "operator" => FilterCriteria::LIKE_ALL
            ));
        }

        $possible_columns = array("name", "labelOnView", "START_DATE_EXPRESSION", "END_DATE_EXPRESSION", "periodsCode");
        if(array_key_exists("in", $params)){
            foreach($params as $key => $value){
                if(in_array($key, $possible_columns)){
                    array_push($search, array(
                        "columnName" => $key,
                        "value" => $value,
                        "operator" => FilterCriteria::LIKE_I
                    ));
                }
            }
        }

        return $search;
    }
}
