import { Metadata, Http } from '@zeedhi/core';
import { Modal, ModalService, Form, Select, Grid, TextInput, AlertService, DialogService, LoadingService, Loading } from '@zeedhi/common';

export class DefaultColumnsController {
    //
    private addModal !: Modal;
    private viewModal !: Modal;
    public viewInputDisable = true;
    //
    public reloadGrid() {
        const grid = Metadata.getInstance<Grid>('defaultcolumns-grid');
        grid.reload();
    }
    //
    public async openAddModal() {
        if (this.addModal !== undefined) {
            await this.addModal?.destroy();
        }
        this.addModal = await ModalService.createFromJson('defaultColumns/addModal/addModal', true);
        this.addModal.show();
    }
    //
    public async closeAddModal() {
        this.addModal.hide();
        await this.addModal?.destroy();
    }
    //
    public async openViewModal() {
        if (this.viewModal !== undefined) {
            await this.viewModal?.destroy();
        }
        this.viewModal = await ModalService.createFromJson('defaultColumns/viewModal/viewModal', true);
        this.viewModal.show();
    }
    //
    public async closeViewModal() {
        this.viewModal.hide();
        await this.viewModal?.destroy();
    }
    //
    public async saveAddModal() {
        const form = Metadata.getInstance<Form>('add-form');
        if (!form.validate()) return;
        const requestBody = {
            disableLoader: false,
            requestType: "DataSet",
            dataset: [
                {
                    ...form.value,
                    changed: true,
                    _createdLocal: true,
                    _is_new: true
                }
            ]
        };
        LoadingService.show('Aguarde');
        await Http.post('defaultColumns/save', requestBody)
        .then(() => {
            this.reloadGrid();
            this.closeAddModal();
            AlertService.show({
                name: "add-alert",
                text: "Padrão criado com sucesso!",
                color: "success"
            });
        }, (reason) => {
            AlertService.show({
                name: "add-alert",
                text: reason,
                color: "error"
            });
        })
        .finally(() => {
            LoadingService.hide();
        });
    }
    //
    public async showRowInfo({component} : any) {
        await this.openViewModal();
        const row = component.datasource.currentRow;
        const form = Metadata.getInstance<Form>('view-form').value;
        this.fillForm(form, row);
        this.viewInputDisable = false;
        // if (row.base === true) {
        //     this.viewInputDisable = true;
        //     DialogService.show({
        //         name: "base-edit-dialog",
        //         type: "info",
        //         title: "Atenção",
        //         text: "Não é possível editar estilos padrões."
        //     });
        // }
    }
    //
    private fillForm(form:any, row:any) {
        form.name = row.name;
        form.label = row.label;
        Metadata.getInstance<Select>('type').setValue(row.type);
        form.isDate = row.isDate;
        Metadata.getInstance<Select>('mask').setValue(row.mask);
        Metadata.getInstance<Select>('direction').setValue(row.direction);
        Metadata.getInstance<Select>('function').setValue(row.function);
        form.isFunctionAccumulative = row.isFunctionAccumulative;
    }
    //
    public async saveViewModal() {
        const form = Metadata.getInstance<Form>('view-form');
        const grid = Metadata.getInstance<Grid>('defaultcolumns-grid');
        const row = grid.datasource.currentRow;
        if (!form.validate()) {
            return;
        }
        const requestBody = {
            disableLoader: false,
            requestType: "DataSet",
            dataset: [
                {
                    ...form.value,
                    changed: true,
                    _createdLocal: true,
                    _is_new: false,
                    _id: row._id
                }
            ]
        };
        LoadingService.show('Aguarde');
        await Http.post('defaultColumns/save', requestBody)
        .then(() => {
            this.reloadGrid();
            this.closeViewModal();
            AlertService.show({
                name: "edit-alert",
                text: "Padrão editado com sucesso!",
                color: "success"
            });
        }, (reason) => {
            AlertService.show({
                name: "edit-alert",
                text: reason,
                color: "error"
            });
        })
        .finally(() => {
            LoadingService.hide();
        });
    }
    //
    public showConfirmDelete({component, event}:any) {
        event.preventDefault();
        DialogService.show({
            name: 'confirm-delete',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o(s) registro(s) selecionado(s)?',
            buttons: [
                { name: 'sim', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteRow(); DialogService.hide(); } }, outline: true },
                { name: 'nao', component: 'ZdButton', label: 'Não', events: { click: () => { DialogService.hide(); } } },
            ],
        });
    }
    //
    private async deleteRow() {
        const grid = Metadata.getInstance<Grid>('defaultcolumns-grid');
        // for (let i = 0; i < grid.selectedRows.length; i += 1) {
        //     if (grid.selectedRows[i].base === true) {
        //         AlertService.show({
        //             name: "delete-alert",
        //             text: "Não é possível deletar estilo(s) padrão(ões)!",
        //             color: "error"
        //         });
        //         return;
        //     }
        // }
        const data = {
            "dataset": grid.selectedRows,
            "disableLoader": false,
            'requestType': 'DataSet'
        };
        LoadingService.show('Aguarde');
        await Http.post('defaultColumns/remove', data)
        .then(() => {
            AlertService.show({
                name: "delete-alert",
                text: "Estilo(s) deletado(s) com sucesso!",
                color: "success"
            });
            this.reloadGrid();
        }, (reason) => {
            AlertService.show({
                name: "delete-alert",
                text: reason,
                color: "error"
            });
        })
        .finally(() => {
            LoadingService.hide();
        });
    }
}
